@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Entry.py,v 1.6 1999/08/18 14:05:32 gobry Exp $
+# $Id: Entry.py,v 1.7 1999/08/25 11:07:35 gobry Exp $
 
 from Pyblio.Base import *
 import string, re
@@ -26,62 +26,193 @@ from gtk import *
 
 from Pyblio import Fields, Config, Base, Types
 
-from Pyblio.GnomeUI import FieldsInfo
+from Pyblio.GnomeUI import FieldsInfo, Utils
 
-def popup_add (menu, item, action = None, argument = None):
-    """ Helper to add a new menu entry """
-    tmp = GtkMenuItem (item)
-    if action:
-        tmp.connect ("activate", action, argument)
-    tmp.show ()
-    menu.append (tmp)
-    
-    return tmp
 
 regular_field = re.compile ("^\w+$")
 regular_id    = re.compile ("^\w[\d\w:_-]+$")
 
+
+class NativeEditor (GtkDialog):
+    """ Implements a dialog that display a simple edition box for native entry """
+    
+    def __init__ (self, entry):
+        
+        self.entry = entry
+        self.modified = None
+        self.native = None
+
+        # do we have our own mainloop ?
+        self.running  = 0
+        
+        GtkDialog.__init__ (self)
+
+        self.set_title ("Native edition of field `%s'" % entry.field)
+        self.set_modal (TRUE)
+        self.set_usize (400, 200)
+        
+        self.connect("delete_event", self.close)
+
+        apply_b = GnomeStockButton (STOCK_BUTTON_APPLY)
+        apply_b.connect ("clicked", self.apply)
+
+        close_b = GnomeStockButton (STOCK_BUTTON_CANCEL)
+        close_b.connect ("clicked", self.close)
+
+        self.action_area.add (apply_b)
+        self.action_area.add (close_b)
+        
+        apply_b.show ()
+        close_b.show ()
+        
+        self.editor = GtkText ()
+        self.editor.set_editable (TRUE)
+        
+        scrolled = GtkScrolledWindow ()
+        scrolled.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
+        scrolled.add (self.editor)
+        
+        self.vbox.pack_start (scrolled)
+
+        if self.entry.entry.has_key (self.entry.field):
+            self.original = self.entry.entry.get_native (self.entry.field)
+            self.editor.insert_defaults (self.original)
+        else:
+            self.original = None
+            
+        scrolled.show_all ()
+        return
+
+    def close (self, * arg):
+        self.destroy ()
+        if self.running: mainquit ()
+        return
+
+    def apply (self, * arg):
+        text = self.editor.get_chars (0, -1)
+        
+        try:
+            self.native = text
+            self.entry.entry.set_native (self.entry.field, text)
+        except IOError, msg:
+            Utils.error_dialog ('Edition error', msg)
+            return
+
+        # get the text corresponding to the entry
+        self.modified = str (self.entry.entry [self.entry.field])
+
+        # reset the old version (we are not supposed to set it now)
+        if self.original is not None:
+            self.entry.entry.set_native (self.entry.field, self.original)
+        else:
+            del self.entry.entry [self.entry.field]
+        
+        self.destroy ()
+        
+        if self.running: mainquit ()
+        return
+
+    def run_and_close (self):
+        self.show ()
+        self.running = 1
+        
+        mainloop ()
+        return self.modified
+    
 class BasicEditor (GtkHBox):
-    def __init__ (self, entry, field):
+    def __init__ (self, entry, field, has_native = 1):
         GtkHBox.__init__ (self, FALSE, 5)
 
         self.entry = entry
         self.field = string.lower (field)
         self.name  = field
         
+        self.editable = 1
+
+        # natively modified ?
+        self.modified = 0
+        # store the eventual native field
+        self.native   = None
+
+        # internal variable to avoid modification notice during self.fill ()
+        self.__in_fill = 0
+        
         if entry.has_key (self.field):
             self.value, loss = entry.text (self.field)
         else:
             self.value, loss = None, 0
 
-        if loss:
-            icon = GnomeStock (STOCK_BUTTON_CANCEL)
-        else:
-            icon = GnomeStock (STOCK_BUTTON_APPLY)
+        # add native editor if we have the native property
+        if has_native:
+            if loss:
+                icon = GnomeStock (STOCK_BUTTON_CANCEL)
+            else:
+                icon = GnomeStock (STOCK_BUTTON_APPLY)
             
-        button = GtkButton ()
-        button.add (icon)
-        button.connect ("clicked", self.edit_native)
+            self.button = GtkButton ()
+            self.button.add (icon)
+            self.button.connect ("clicked", self.edit_native)
         
-        self.pack_end (button, FALSE, FALSE)
+            self.pack_end (self.button, FALSE, FALSE)
+            
         self.show_all ()
+
         return
 
-
+    def modified_cb (self, * arg):
+        """ called when the textual entry is modified. This discards the
+        native edition """
+        
+        if not self.__in_fill:
+            self.modified = 0
+            
+        return
+    
+    def set_editable (self, val):
+        self.editor.set_editable (val)
+        self.button.set_sensitive (val)
+        
+        self.editable = val
+        return
+        
     def edit_native (self, * arg):
+        n = NativeEditor (self)
+        
+        # was the entry modified ?
+        text = n.run_and_close ()
+        
+        if text is not None:
+            self.modified = 1
+            self.native   = n.native
+
+            # display the modified text
+            self.fill (text)
         return
+    
+    
+    def fill (self, text = None):
+        self.__in_fill = 1
+        
+        text = text or self.value
+        
+        if text is None:
+            self.editor.set_text ('')
+        else:
+            self.editor.set_text (str (text))
 
-
-    def fill (self):
-        if self.value:
-            self.editor.set_text (str (self.value))
+        self.__in_fill = 0
         return
 
     
     def update (self):
         new = self.get ()
+
+        # entry has been natively modified
+        if self.modified:
+            self.entry.set_native (self.field, self.native)
+            return 1
+        
         if new != self.value:
-            
             if new:
                 self.entry [self.field] = new
             else:
@@ -103,8 +234,8 @@ class BasicEditor (GtkHBox):
     
 class TextEditor (BasicEditor):
     
-    def __init__ (self, entry, field):
-        BasicEditor.__init__ (self, entry, field)
+    def __init__ (self, entry, field, has_native = 1):
+        BasicEditor.__init__ (self, entry, field, has_native)
         
         scrolled = GtkScrolledWindow ()
         scrolled.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
@@ -113,15 +244,23 @@ class TextEditor (BasicEditor):
         self.editor.set_editable (TRUE)
         self.editor.set_usize (300,100)
         self.editor.set_word_wrap (1)
-
+        self.editor.connect ("changed", self.modified_cb)
+        
         scrolled.add (self.editor)
         self.pack_start (scrolled, TRUE, TRUE)
         self.fill ()
         return
 
-    def fill (self):
-        if self.value:
-            self.editor.insert_defaults (str (self.value))
+    def fill (self, text = None):
+        text = text or self.value
+        
+        self.editor.freeze ()
+        self.editor.delete_text (0, -1)
+
+        if text is not None:
+            self.editor.insert_defaults (str (text))
+            
+        self.editor.thaw ()
         return
 
     def get (self):
@@ -133,13 +272,14 @@ class TextEditor (BasicEditor):
 
 class AuthorEditor (BasicEditor):
     
-    def __init__ (self, entry, field):
-        BasicEditor.__init__ (self, entry, field)
+    def __init__ (self, entry, field, has_native = 1):
+        BasicEditor.__init__ (self, entry, field, has_native)
 
         scrolled = GtkScrolledWindow ()
         scrolled.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
         
         self.editor = GtkText ()
+        self.editor.connect ("changed", self.modified_cb)
         self.editor.set_editable (TRUE) 
         self.editor.set_word_wrap (1)
         self.editor.set_usize (300,100)
@@ -149,7 +289,7 @@ class AuthorEditor (BasicEditor):
         self.fill ()
         return
 
-    def fill (self):
+    def fill (self, text = None):
         def str_auth (auth):
             if auth.first:
                 ret = auth.last + ', ' + auth.first
@@ -160,8 +300,15 @@ class AuthorEditor (BasicEditor):
 
             return ret
 
-        if self.value:
-            self.editor.insert_defaults (join (map (str_auth, self.value), "\n"))
+        text = text or self.value
+        
+        self.editor.freeze ()
+        self.editor.delete_text (0, -1)
+        
+        if text is not None:
+            self.editor.insert_defaults (join (map (str_auth, text), "\n"))
+            
+        self.editor.thaw ()
         return
 
     def get (self):
@@ -179,9 +326,11 @@ class AuthorEditor (BasicEditor):
     
 class FieldEditor (BasicEditor):
     
-    def __init__ (self, entry, field):
-        BasicEditor.__init__ (self, entry, field)
+    def __init__ (self, entry, field, has_native = 1):
+        BasicEditor.__init__ (self, entry, field, has_native)
+        
         self.editor = GtkEntry ()
+        self.editor.connect ("changed", self.modified_cb)
         self.editor.set_editable (TRUE)
         self.pack_start (self.editor, TRUE, TRUE)
         self.fill ()
@@ -191,9 +340,11 @@ class FieldEditor (BasicEditor):
 
 class DateEditor (BasicEditor):
     
-    def __init__ (self, entry, field):
-        BasicEditor.__init__ (self, entry, field)
+    def __init__ (self, entry, field, has_native = 1):
+        BasicEditor.__init__ (self, entry, field, has_native)
+        
         self.editor = GtkEntry ()
+        self.editor.connect ("changed", self.modified_cb)
         self.editor.set_editable (TRUE)
         self.pack_start (self.editor, TRUE, TRUE)
         self.fill ()
@@ -280,7 +431,7 @@ class Entry (GtkVBox):
     """ Entry displayer """
     
     def __init__ (self, entry, properties):
-        GtkVBox.__init__ (self, FALSE, 5)
+        GtkVBox.__init__ (self, FALSE)
         
         self.entry      = entry
         self.properties = properties
@@ -300,15 +451,16 @@ class Entry (GtkVBox):
 
         self.name = GtkEntry ()
 
-        if self.properties ['change_type']:
+        if self.has_property ('change_type'):
             self.type = GtkOptionMenu ()
             available = map (lambda x: x.name, Config.get ("base/entries").data.values ())
+            available.sort ()
             
             menu = GtkMenu ()
             self.type.set_menu (menu)
             
             for avail in available:
-                popup_add (menu, avail, self.type_changed, avail)
+                Utils.popup_add (menu, avail, self.type_changed, avail)
 
             self.type.set_history (available.index (ent.name))
         else:
@@ -319,7 +471,7 @@ class Entry (GtkVBox):
         
         self.name.set_text (entry.name)
 
-        if not self.properties ['change_id']:
+        if not self.has_property ('change_id'):
             self.name.set_editable (FALSE)
 
 
@@ -328,7 +480,37 @@ class Entry (GtkVBox):
             
         table.show_all ()
 
-        if self.properties ['has_extra']:
+        if self.has_property ('crossref'):
+            # crossref handling
+            box = GtkHBox (FALSE, 5)
+            box.set_border_width (5)
+
+            box.pack_start (GtkLabel ('Cross-Reference:'), fill=FALSE, expand = FALSE)
+            
+            self.crossref = GtkEntry ()
+            self.crossref.set_editable (FALSE)
+            
+            box.pack_start (self.crossref)
+
+            button = GtkButton ()
+            button.add (GnomeStock (STOCK_BUTTON_CANCEL))
+            button.connect ("clicked", self.remove_crossref)
+            
+            box.pack_start (button, fill = FALSE, expand = FALSE)
+            
+            self.pack_start (box, fill=FALSE, expand=FALSE)
+
+            # initialize the field
+            if self.entry.crossref:
+                self.crossref.set_text (str (self.entry.crossref.key.key))
+
+            targets = [ ('application/x-pybkey', 0, 1) ]
+            
+            self.crossref.connect ('drag_data_received', self.dnd_drag_data_received)
+            self.crossref.drag_dest_set(DEST_DEFAULT_ALL, targets, GDK.ACTION_COPY)
+
+
+        if self.has_property ('has_extra'):
             # extra field addition
             hbox   = GtkHBox (FALSE, 5)
             hbox.set_border_width (5)
@@ -340,9 +522,29 @@ class Entry (GtkVBox):
 
             add_button.connect ("clicked", self.extra_field, select)
 
+            
         self.create_notebook (ent)
         return
+
+
+    def dnd_drag_data_received (self, w, context, x, y, data, info, time):
+        self.crossref.set_text (data.data)
+        return
+
+
+    def remove_crossref (self, * arg):
+        self.crossref.set_text ("")
+        return
     
+    def has_property (self, prop):
+        """ indicates if the database has a given property """
+		
+        if self.properties.has_key (prop):
+            return self.properties [prop]
+        
+        return 0
+
+
     def create_notebook (self, ent):
         
         # Notebook holding Mandatory/Optional fields
@@ -351,26 +553,33 @@ class Entry (GtkVBox):
         self.mandatory = FieldGroup (ent.mandatory)
         self.optional  = FieldGroup (ent.optional)
         self.extra     = FieldGroup ()
+
+        native = self.has_property ('native')
         
-        def get_field (entry, field):
+        def get_field (entry, field, native = native):
             type = FieldsInfo.fieldinfo (field).type
-            
+
             if type == FieldsInfo.WidgetText: 
-                tmp = TextEditor (entry, field)
+                tmp = TextEditor (entry, field, native)
 
             elif type == FieldsInfo.WidgetEntry:
-                tmp = FieldEditor (entry, field)
+                tmp = FieldEditor (entry, field, native)
 
             elif type == FieldsInfo.WidgetAuthor:
-                tmp = AuthorEditor (entry, field)
+                tmp = AuthorEditor (entry, field, native)
 
             elif type == FieldsInfo.WidgetDate:
-                tmp = DateEditor (entry, field)
+                tmp = DateEditor (entry, field, native)
 
             else:
-                tmp = TextEditor (entry, field)
+                tmp = TextEditor (entry, field, native)
 
             tmp.show ()
+
+            if (tmp.value is not None) and not entry.is_personal (string.lower (field)):
+                # for cross-referenced fields
+                tmp.set_editable (FALSE)
+                
             return tmp
         
         dico = self.entry.keys ()
@@ -434,6 +643,7 @@ class EntryDialog (GtkDialog):
     def __init__ (self, entry, database, callback = None, user = None):
 
         GtkDialog.__init__ (self)
+        self.set_policy (TRUE, TRUE, TRUE)
         
         self.set_title ('Edit')
         self.connect("delete_event", self.close)
@@ -462,8 +672,8 @@ class EntryDialog (GtkDialog):
         changed = self.entry.changed
         
         changed = self.entry.mandatory.update () or changed
-        changed = self.entry.optional.update () or changed
-        changed = self.entry.extra.update () or changed
+        changed = self.entry.optional.update ()  or changed
+        changed = self.entry.extra.update ()     or changed
 
         # check the id
         id = string.strip (self.entry.name.get_text ())
@@ -483,7 +693,18 @@ class EntryDialog (GtkDialog):
                 d = GnomeErrorDialog ("Identifier `%s' already exists" % id)
                 d.run_and_close ()
                 return
+
+        # check the crossref
+        cr = self.entry.crossref.get_text ()
+        if cr == '': cr = None
         
+        oldcr = self.entry.entry.crossref
+        if oldcr: oldcr = oldcr.key
+        
+        if oldcr != cr:
+            changed = 1
+            self.entry.entry.crossref = self.database [Base.Key (self.database, cr)]
+            
         if changed and self.callback:
             self.callback (self.entry.entry, self.user)
         
