@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Pybliographic.py,v 1.13 1999/08/25 11:07:35 gobry Exp $
+# $Id: Pybliographic.py,v 1.15 1999/08/26 11:16:04 gobry Exp $
 
 import string, os
 
@@ -49,19 +49,21 @@ class URLFileSelection (GtkFileSelection):
         self.ok_button.connect('clicked', self.ok_cb)
 
         if modal:
-            grab_add(self)
+            grab_add (self)
 
         self.ret = None
-
+        self.url = None
+        
         vbox = self.main_vbox
         
         # url handler
-        hbox = GtkHBox ()
-        hbox.set_spacing (5)
-        hbox.pack_start (GtkLabel ("URL:"), expand = FALSE, fill = FALSE)
-        self.url = GtkEntry ()
-        hbox.pack_start (self.url)
-        vbox.pack_start (hbox, expand = FALSE, fill = FALSE)
+        if url:
+            hbox = GtkHBox ()
+            hbox.set_spacing (5)
+            hbox.pack_start (GtkLabel ("URL:"), expand = FALSE, fill = FALSE)
+            self.url = GtkEntry ()
+            hbox.pack_start (self.url)
+            vbox.pack_start (hbox, expand = FALSE, fill = FALSE)
 
         # type selector
         hbox = GtkHBox ()
@@ -75,14 +77,19 @@ class URLFileSelection (GtkFileSelection):
         # menu content
         menu = GtkMenu ()
         self.menu.set_menu (menu)
+        
+        liste = Open.available ()
+
         if has_auto:
             Utils.popup_add (menu, " - Auto - ", self.menu_select, None)
-
-        for avail in Open.available ():
-            Utils.popup_add (menu, avail, self.menu_select, avail)
+            self.type = None
+        else:
+            self.type = liste [0]
             
+        for avail in liste:
+            Utils.popup_add (menu, avail, self.menu_select, avail)
+
         self.menu.set_history (0)
-        self.type = None
         return
 
     def menu_select (self, widget, selection):
@@ -97,9 +104,14 @@ class URLFileSelection (GtkFileSelection):
     
     def ok_cb(self, b):
         self.ret = self.get_filename()
+        
         if self.ret [-1] == '/':
-            self.ret = self.url.get_text ()
-            if self.ret == "": self.ret = None
+            if self.url:
+                self.ret = self.url.get_text ()
+                if self.ret == "": self.ret = None
+            else:
+                self.ret = None
+        
         self.quit()
         return
     
@@ -217,10 +229,9 @@ class Pybliographic (GnomeApp):
 
         self.main_box.pack_start (paned, expand=TRUE)
 
-        self.status = GtkStatusbar ()
+        self.status = GnomeAppBar (FALSE)
         self.set_statusbar (self.status)
-        self.status.show ()
-        self.status.push (1, "Welcome to Pybliographic")
+        self.status.push ("Welcome to Pybliographic")
 
         # Le popup menu
         self.menu = GtkMenu ()
@@ -237,6 +248,8 @@ class Pybliographic (GnomeApp):
 
         self.directory = './'
         self.modified  = 0
+
+        self.search = None
         return
     
     # --------------------------------------------------
@@ -265,9 +278,11 @@ class Pybliographic (GnomeApp):
             Utils.error_dialog ('Creation error', err)
             return
 
-        self.status.pop (1)
-        self.status.push (1, "Database `%s' with %d entries"
-                          % (file, len (self.database.data.keys ())))
+        self.filename = file
+        self.type     = type
+        
+        self.status.pop ()
+        self.status.push ("Database `%s' [%s]" % (self.filename, self.type))
         return
     
     # --------------------------------------------------
@@ -287,9 +302,13 @@ class Pybliographic (GnomeApp):
             Utils.error_dialog ('Open Error', err)
             return
 
-        self.status.pop (1)
-        self.status.push (1, "Database `%s' with %d entries"
-                          % (file, len (self.database.data.keys ())))
+        self.status.pop ()
+        
+        self.filename = file
+        self.type     = self.database.data.id
+        
+        self.status.push ("Database `%s' [%s, %d entries]"
+                   % (self.filename, self.type, len (self.database.data.keys ())))
 
 
     def file_open (self, *arg):
@@ -324,10 +343,12 @@ class Pybliographic (GnomeApp):
     # --------------------------------------------------
 
     def file_save (self, *arg):
+        if not self.modified: return
+        
         try:
             self.database.data.update ()
-            self.status.pop (1)
-            self.status.push (1, "Database saved")
+            self.status.pop ()
+            self.status.push ("Database saved")
             self.modified = 0
             
         except IOError, err:
@@ -338,32 +359,67 @@ class Pybliographic (GnomeApp):
     # --------------------------------------------------
 
     def file_save_as (self, *arg):
-        win = URLFileSelection ("Save", FALSE, TRUE)
+        win = URLFileSelection ("Save As", FALSE, TRUE, FALSE)
         win.set_filename (self.directory)
 
         file = win.run ()
+        if not file: return
+        
+        if os.path.exists (file):
+            var = []
+                
+            def callback (arg, var = var):
+                if arg == 0: var.append (1)
+                return
 
+            d = GnomeQuestionDialog ("File `%s' exists.\nOverwrite it ?" % file,
+                                     callback)
+            d.run_and_close ()
+
+            if not var: return
+                
+        # write the file
+        try:
+            fh = open (file, 'w')    
+            Open.bibwrite (self.database.data, how = win.type, out = fh)
+            fh.close ()
+        except IOError, msg:
+            d = GnomeErrorDialog ("Can't save file:\n%s" % msg)
+            d.run_and_close ()
+            return
+
+        self.open (file, win.type)
+        return
+    
     # --------------------------------------------------
     # --------------------------------------------------
 
     def edit_copy (self, * arg):
-        pass
+        d = GnomeOkDialog ("Not yet implemented...")
+        d.show ()
+        return
 
     # --------------------------------------------------
 
     def edit_cut (self, * arg):
-        pass
+        d = GnomeOkDialog ("Not yet implemented...")
+        d.show ()
+        return
 
     # --------------------------------------------------
 
     def edit_paste (self, * arg):
-        pass
+        d = GnomeOkDialog ("Not yet implemented...")
+        d.show ()
+        return
     
     # --------------------------------------------------
 
     def edit_search (self, * arg):
-        search = SearchDialog (self.database)
-        search.show_all ()
+        if self.search is None:
+            self.search = SearchDialog (self.database)
+        
+        self.search.show_all ()
         return
     
     # --------------------------------------------------
@@ -384,10 +440,10 @@ class Pybliographic (GnomeApp):
         entry = self.database.data.new_entry ()
         
         def callback (entry, self, selection = row):
-            self.status.pop (1)
-            self.status.push (1, "...modified...")
+            self.status.pop ()
+            self.status.push ("...modified...")
             self.modified = 1
-            entry.key = Base.Key (self.database.data, entry.name)
+            
             self.database.data [entry.key] = entry
             self.database.insert (selection, entry)
             return
@@ -412,8 +468,8 @@ class Pybliographic (GnomeApp):
         e = selection [0]
         
         def callback (entry, self, selection = row):
-            self.status.pop (1)
-            self.status.push (1, "...modified...")
+            self.status.pop ()
+            self.status.push ("...modified...")
             self.modified = 1
             
             self.database.select_row (selection, 0)
@@ -447,15 +503,18 @@ class Pybliographic (GnomeApp):
         if self.__ans:
             self.database.remove (entry.key)
             self.modified = 1
-            self.status.pop (1)
-            self.status.push (1, "...modified...")
+            
+            self.status.pop ()
+            self.status.push ("...modified...")
         return
     
         
     # --------------------------------------------------
 
     def configure_prefs (self, * arg):
-        pass
+        d = GnomeOkDialog ("Not yet implemented...")
+        d.show ()
+        return
 
     # --------------------------------------------------
 
