@@ -18,7 +18,7 @@
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
- $Id: bibtex.h,v 1.21 1999/07/20 14:53:47 gobry Exp $
+ $Id: bibtex.h,v 1.24 1999/08/16 09:38:29 gobry Exp $
 */
 
 #ifndef __bibtex_h__
@@ -148,6 +148,8 @@ extern "C" {
 	int length;
 	int offset;
 
+	int start_line;
+
 	BibtexStruct * preamble;
 
 	GHashTable * table;
@@ -158,37 +160,66 @@ extern "C" {
       Full BibTeX database
     */
 
+    typedef enum {
+	BIBTEX_SOURCE_NONE,
+	BIBTEX_SOURCE_FILE,
+	BIBTEX_SOURCE_STRING
+    }
+    BibtexSourceType;
+
     typedef struct {
-	gchar * filename;
-	FILE * file;
+	BibtexSourceType type;
+
+	gchar * name;
+
+	union {
+	    FILE  * file;
+	    gchar * string;
+	} source;
 
 	int line;
 	int offset;
 
-
 	GHashTable * table;
 	gpointer buffer;
 
 	int debug;
 	gboolean eof, error;
     }
-    BibtexFile;
+    BibtexSource;
 
     /* -------------------------------------------------- 
        High level interface
        -------------------------------------------------- */
 
-    BibtexFile * bibtex_open  (gchar * file);
-    void         bibtex_close (BibtexFile *);
-
-    BibtexEntry * bibtex_next_entry     (BibtexFile * file);
-    void    	  bibtex_first_entry    (BibtexFile * file);
-    void    	  bibtex_offset_entry   (BibtexFile * file, gint);
-    gint    	  bibtex_current_offset (BibtexFile * file);
-
     BibtexEntry * bibtex_entry_new      (void);
-    void          bibtex_entry_destroy  (BibtexEntry * entry, gboolean content);
 
+    void          bibtex_entry_destroy  (BibtexEntry * entry, 
+					 gboolean content);
+
+
+    /* Source manipulation */
+
+    BibtexSource * bibtex_source_new (void);
+
+    void           bibtex_source_destroy (BibtexSource * source, 
+					  gboolean free_data);
+
+    gboolean       bibtex_source_file (BibtexSource * source, gchar *
+				       filename);
+
+    gboolean       bibtex_source_string (BibtexSource * source, 
+					 gchar * name,
+					 gchar * string);
+
+    BibtexEntry *  bibtex_source_next_entry (BibtexSource * file, gboolean filter);
+
+    void           bibtex_source_rewind (BibtexSource * file);
+
+    gint           bibtex_source_get_offset (BibtexSource * file);
+    
+    void           bibtex_source_set_offset (BibtexSource * file, 
+					     gint offset);
 
     /* Fields manipulation */
 
@@ -219,7 +250,7 @@ extern "C" {
     BibtexStruct * bibtex_struct_append  (BibtexStruct *, BibtexStruct *);
 
 
-    /* Structure convertions */
+    /* Structure conversions */
 
     gchar *       bibtex_struct_as_string (BibtexStruct * s, 
 					   BibtexFieldType type, 
@@ -231,8 +262,10 @@ extern "C" {
     BibtexField * bibtex_struct_as_field  (BibtexStruct * s, 
 					   BibtexFieldType type);
 
-    BibtexField * bibtex_string_as_field (gchar * string,
-					  BibtexFieldType type);
+
+    /* recreate structure after modifications */
+
+    BibtexField * bibtex_reverse_field (BibtexField * field);
 
     /* --------------------------------------------------
        Low level function
@@ -246,11 +279,11 @@ extern "C" {
     void    bibtex_tmp_string_free (void);
     
     /* Parse next entry */
-    BibtexEntry * parse_bibtex (BibtexFile * file);
+    BibtexEntry * bibtex_analyzer_parse (BibtexSource * file);
 
     /*  analyser on a file */
-    void bibtex_analyzer_initialize (BibtexFile * file);
-    void bibtex_analyzer_finish     (BibtexFile * file);
+    void bibtex_analyzer_initialize (BibtexSource * file);
+    void bibtex_analyzer_finish     (BibtexSource * file);
 
 #ifdef __cplusplus
 }
