@@ -155,3 +155,71 @@ Config.set ("gnomeui/default",
             FieldsInfo.UIDescription (150, FieldsInfo.WidgetEntry))
 
 Config.set ("gnomeui/columns", ('Author', 'Year', 'Title'))
+
+# ==================================================
+
+Config.set ("endnotes/table", {
+    'U' : 'url',
+    'A' : 'author',
+    'Q' : 'author',
+    'T' : 'title',
+    'S' : 'series',
+    'J' : 'journal',
+    'B' : 'booktitle',
+    'R' : 'type',
+    'V' : 'volume',
+    'N' : 'number',
+    'E' : 'editor',
+    'D' : 'year',
+    'P' : 'pages',
+    'I' : 'publisher',
+    'C' : 'address',
+    'K' : 'keywords',
+    'X' : 'abstract',
+    'W' : 'location',
+    '0' : ' type ',
+    '7' : 'edition',
+    '8' : 'confdate',
+    '9' : 'howpublished',
+    'Y' : 'editor',
+    })
+
+Config.set ("refer/table", {
+    'U' : 'url',
+    'A' : 'author',
+    'Q' : 'author',
+    'T' : 'title',
+    'S' : 'series',
+    'J' : 'journal',
+    'B' : 'booktitle',
+    'R' : 'type',
+    'V' : 'volume',
+    'N' : 'number',
+    'E' : 'editor',
+    'D' : 'year',
+    'P' : 'pages',
+    'I' : 'publisher',
+    'C' : 'address',
+    'K' : 'keywords',
+    'X' : 'abstract',
+    'W' : 'location',
+    })
+
+Config.set ("endnotes/types", {
+    "Artwork" : 'misc',
+    "Audiovisual Material" : 'misc',
+    "Book" : 'book',
+    "Book Section" : 'inbook',
+    "Computer Program" : 'manual',
+    "Conference Proceedings" : 'proceedings',
+    "Edited Book" : 'book',
+    "Generic" : 'misc',
+    "Journal Article" : 'article',
+    "Magazine Article" : 'article',
+    "Map" : 'misc',
+    "Newspaper Article" : 'article',
+    "Patent" : 'misc',
+    "Personal Communication" : 'misc',
+    "Report" : 'techreport',
+    "Thesis" : 'phdthesis',
+    })
